@@ -18,6 +18,7 @@ from pymatgen.analysis.local_env import cn_opt_params
 from pymatgen.analysis.structure_matcher import StructureMatcher
 from pymatgen.core import Lattice, Structure
 from pymatgen.io.vasp import VolumetricData
+from pymatgen.io.vasp.sets import get_valid_magmom_struct
 from scipy.cluster.hierarchy import fcluster, linkage
 from scipy.spatial.distance import squareform
 
@@ -395,16 +396,18 @@ class ChargeInsertionAnalyzer(MSONable):
     Analyze the charge density and create new candidate structures by inserting at each charge minima
     The similar inserterd structures are given the same uniqueness label.
 
-    **Warning**: This works best with AECCAR data since CHGCAR data often contains spurious local minima in the core.
-    However you can still use CHGCAR with an appropriate max_avg_charge value.
+    .. note::
+        The charge density analysis works best with AECCAR data since CHGCAR data
+        often contains spurious local minima in the core. However you can still use CHGCAR
+        with an appropriate ``max_avg_charge`` value.
 
-    **Note**: Since the user might want to rerun their analysis with different `avg_charge` and `max_avg_charge` values,
-    we will generate and store all the ion-inserted structure and their uniqueness labels first and allow the user to
-    get the filtered and labeled results.
+        Since the user might want to rerun their analysis with different ``avg_charge`` and ``max_avg_charge`` values,
+        we will generate and store all the ion-inserted structure and their uniqueness labels first and allow
+        the user to get the filtered and labeled results.
 
-    If you use this code please cite the following paper:
-    J.-X. Shen et al.: npj Comput. Mater. 6, 1 (2020)
-    https://www.nature.com/articles/s41524-020-00422-3
+        If you use this code please cite the following paper:
+        J.-X. Shen et al.: npj Comput. Mater. 6, 1 (2020)
+        https://www.nature.com/articles/s41524-020-00422-3
     """
 
     def __init__(
@@ -441,7 +444,7 @@ class ChargeInsertionAnalyzer(MSONable):
         2. Group the candidate sites by symmetry
         3. Label the groups by structure matching
         4. Since the average charge density is the most expensive part,
-            we will leave this until the end.
+        we will leave this until the end.
 
         Returns:
             list[tuple[list[float], int]]: A list of tuples of the form (fcoords, label)
@@ -459,11 +462,11 @@ class ChargeInsertionAnalyzer(MSONable):
         inserted_structs = []
         for fpos in local_minima:
             tmp_struct = self.chgcar.structure.copy()
+            get_valid_magmom_struct(tmp_struct, inplace=True, spin_mode="none")
             tmp_struct.insert(
                 0,
                 self.working_ion,
                 fpos,
-                properties=dict(magmom=0),
             )
             tmp_struct.sort()
             inserted_structs.append(tmp_struct)
@@ -479,16 +482,16 @@ class ChargeInsertionAnalyzer(MSONable):
 
     def filter_and_group(
         self, avg_radius: float = 0.4, max_avg_charge: float = 1.0
-    ) -> list[tuple[float, list[int]]]:
-        """Filter and group the inserted structures.
+    ) -> list[tuple[float, list[list[float]]]]:
+        """Filter and group the insertion sites by average charge.
 
         Args:
             avg_radius: The radius used to calculate average charge density.
             max_avg_charge: Do no consider local minmas with avg charge above this value.
 
         Returns:
-            list[tuple[float, list[int]]]: The list of `(avg_charge, index_group)` tuples
-            where `index_group` are the indices of `self.local_minima` that are in the group.
+            list[tuple[float, list[int]]]: The list of ``(avg_charge, site_group)`` tuples
+            where ``site_group`` are the positions of the local minima.
         """
         # measure the charge density at one representative site of each group
         lab_groups = collections.defaultdict(list)
@@ -498,13 +501,13 @@ class ChargeInsertionAnalyzer(MSONable):
         avg_chg_first_member = {}
         for lab, g in lab_groups.items():
             avg_chg_first_member[lab] = get_avg_chg(
-                self.chgcar, fcoord=self.labeled_sites[g[0]][0], radius=avg_radius
+                self.chgcar, fcoord=self.local_minima[g[0]], radius=avg_radius
             )
 
         res = []
         for lab, avg_chg in sorted(avg_chg_first_member.items(), key=lambda x: x[1]):
             if avg_chg > max_avg_charge:
                 break
-            res.append((avg_chg, lab_groups[lab]))
+            res.append((avg_chg, [self.local_minima[idx] for idx in lab_groups[lab]]))
 
         return res
